################################################################################
## Consumption Capital Asset Pricing Model
## Data for Estimation
## Last modified: Sat Apr  6 17:04:35 2013
##
## Authors: Nicola Borri and Giuseppe Ragusa
################################################################################

library(xts)
library(R.matlab)
library(tis)
################################################################################
## Consumption data
##
## Downloaded: "Fri Apr  5 12:16:24 2013"
## From: Datastream
##  "C.Q"        - Real Consumption per capita (Nondurable plus service) '000 $
##  "Pindex.NDS" - Price index (Nondurable plus service)
################################################################################
filename  <- "~/Dropbox/Research/CCAPM/MyReplicationFilesYogo/US_NDC_Q.mat"
if(file.exists(filename)==TRUE){cc <- readMat(filename)}else{cc <- readMat("~/Dropbox/CCAPM/MyReplicationFilesYogo/US_NDC_Q.mat")}
nds <- ts(log(cc[[1]][,-1]), start = c(1950,1), frequency = 4)
pnds <- ts(cc[[2]][,-1], start = c(1950,1), frequency = 4)
nds <- window(nds, start=c(1959,1))
pnds <- window(pnds, start=c(1959,1))
pnds <- pnds/c(window(pnds, start= c(2005,1), end=c(2005,1)))
################################################################################
## Fama French Factors 
##
## The Fama/French factors are constructed using the 6
## value-weight portfolios formed on size and book-to-market. 
##
## V1    - Monthly Nominal Excess Returns (Rm-Rf)
## V2-V3 - Monthly 2 factors
## V5    - Nominal T-bill monthly return
##
## Downloaded: "Fri Apr  5 12:16:24 2013" From: 
## http://mba.tuck.dartmouth.edu/pages/faculty/ken.french/ftp/F-F_Research_Data_Factors.zip
##
################################################################################
filename  <- "~/Dropbox/Research/CCAPM/MyReplicationFilesYogo/to_update/F-F_Research_Data_Factors.txt"
if(file.exists(filename)==TRUE){
  fff <- read.table(filename,skip=4,nrows=1039)
}
  else{
    fff <- read.table('~/Dropbox/CCAPM/MyReplicationFilesYogo/to_update/F-F_Research_Data_Factors.txt', skip = 4, nrows=1039)}

fff <- ts(fff[,-1], start=c(1926, 7), frequency = 12)
fff <- as.xts(fff)
rf <- xts:::apply.quarterly(fff[,4], FUN=function(u) apply(u, 2, function(uu) prod(1+uu/100)))
rf <- ts(as.matrix(rf)-1, start = c(1926,3), frequency=4)
rf <- window(rf, start = c(1959,1), end = c(2012,4))

er <- apply.quarterly(fff[,1], FUN=function(u) apply(u, 2, function(uu) prod(1+uu/100)))
er <- ts(as.matrix(er)-1, start = c(1926,3), frequency=4)
er <- window(er, start = c(1959,1), end = c(2012,4))

################################################################################
## Fama French (6 Portfolios Formed on Size and Book-to-Market (2 x 3))
##
## Downloaded: "Fri Apr  5 12:16:24 2013"
## From: 
## mba.tuck.dartmouth.edu/pages/faculty/ken.french/ftp/6_Portfolios_2x3.zip
##
################################################################################
filename  <- "~/Dropbox/Research/CCAPM/MyReplicationFilesYogo/to_update/6_Portfolios_2x3.txt"
if(file.exists(filename)==TRUE){portfolio <- read.table(filename,skip=20,nrows=1039)}else{portfolio <- read.table('~/Dropbox/CCAPM/MyReplicationFilesYogo/to_update/6_Portfolios_2x3.txt', skip = 20, nrows=1039)}
portfolio <- ts(portfolio, start = c(1926, 7), frequency = 12 )
colnames(portfolio) <- c(paste0('low.', 1:3), paste0('high.', 1:3))
portfolio <- as.xts(portfolio)
qtr_portfolios <- xts:::apply.quarterly(portfolio, FUN=function(u) apply(u, 2, function(uu) prod(1+uu/100)))
ffportfolios_6 <- window(ts(as.matrix(qtr_portfolios-1), start = c(1926,3), frequency=4),
                         start = c(1959,1), end = c(2012,4))


################################################################################
## Fama French (25 Portfolios Formed on Size and Book-to-Market (5 x 5))
##
## Downloaded: "Fri Apr  5 12:18:24 2013"
## From: 
## http://mba.tuck.dartmouth.edu/pages/faculty/ken.french/ftp/25_Portfolios_5x5.zip
##
################################################################################
filename  <- "~/Dropbox/Research/CCAPM/MyReplicationFilesYogo/to_update/25_Portfolios_5x5.txt"
if(file.exists(filename)==TRUE){portfolio <- read.table(filename,skip=20,nrows=1038)}else{portfolio <- read.table('~/Dropbox/CCAPM/MyReplicationFilesYogo/to_update/25_Portfolios_5x5.txt', skip = 20, nrows=1038)}
portfolio <- ts(portfolio, start = c(1926, 7), frequency = 12 )

colnames(portfolio) <- paste0("P_",1:25, "_25")
portfolio <- as.xts(portfolio)
qtr_portfolios <- xts:::apply.quarterly(portfolio, FUN=function(u) apply(u, 2, function(uu) prod(1+uu/100)))
ffportfolios_25 <- window(ts(as.matrix(qtr_portfolios-1), start = c(1926,3), frequency=4),
                          start = c(1959,1), end = c(2012,4))



################################################################################
## Create Data for different CCAPM versios
##
## 
################################################################################

infl <- diff(log(pnds))
inflt1 <- lag(infl, -1)
rrf <- rf-infl
rrm <- (er+rf)-infl
rer <- rrm-rrf
rff_portfolios_6 <- ts(apply(ffportfolios_6, 2, function(u) u/pnds), start = c(1959,1), frequency = 4)
rff_portfolios_25 <- ts(apply(ffportfolios_25, 2, function(u) u/pnds), start = c(1959,1), frequency = 4)
Dc <- diff(log(nds))
Dct1 <- lag(Dc,-1)
rft1 <- lag(rf, -1)
pu_capm <- cbind(Dc, rrf, rrm, rer, infl, Dct1, rft1, inflt1, rff_portfolios_6, rff_portfolios_25)
colnames(pu_capm) <- c("Dc", "rf", "rm", "er", "infl", "Dct1", "rft1", "inflt1", paste0("p6_",1:6), paste0('p25_', 1:25))
save(pu_capm, file = 'pu_capm.RData')
